"""Kurator.py

This module demonstrates documentation as specified by the `Google Python
Style Guide`_. Docstrings may extend over multiple lines. Sections are created
with a section header and a colon followed by a block of indented text.

Example:
    Examples can be given using either the ``Example`` or ``Examples``
    sections. Sections support any reStructuredText formatting, including
    literal blocks::

        $ python example_google.py

Section breaks are created by resuming unindented text. Section breaks
are also implicitly created anytime a new section starts.

Attributes:
    module_level_variable1 (int): Module level variables may be documented in
        either the ``Attributes`` section of the module docstring, or in an
        inline docstring immediately following the variable.

        Either form is acceptable, but the two should not be mixed. Choose
        one convention to document module level variables and be consistent
        with it.

Todo:
    * For module TODOs
    * You have to also use ``sphinx.ext.todo`` extension

.. _Google Python Style Guide:
   http://google.github.io/styleguide/pyguide.html

"""

import os
import sys
import filecmp
import logging
from shutil import copy2 as copy_file, SameFileError

from kurator.lib import utils as u

class NoError(logging.Filter):
    """Only reports items not in Warning, Error, and Critical
    """
    def filter(self, record):
        return not record.levelname in ['WARNING', 'ERROR', 'CRITICAL']

LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.INFO)

FORMATTER = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')

HANDLER_STDOUT = logging.StreamHandler(sys.stdout)
HANDLER_STDOUT.setLevel(logging.WARNING)
HANDLER_STDOUT.setFormatter(FORMATTER)
HANDLER_STDOUT.addFilter(NoError())

HANDLER_STDERR = logging.StreamHandler()
HANDLER_STDERR.setLevel(logging.INFO)
HANDLER_STDERR.setFormatter(FORMATTER)

LOGGER.addHandler(HANDLER_STDOUT)
LOGGER.addHandler(HANDLER_STDERR)

def import_media(source, library):
    """ Imports media from source into library """

    files = u.find_all_files(source, ('.jpg', '.mp4', '.mov'))
    print('Processing {} photos'.format(len(files)))
    for idx, file_item in enumerate(files):
        folder_name = os.path.join(library, u.generate_foldername_from_meta(file_item))
        file_name = u.generate_filename_from_meta(file_item)
        u.create_directory(folder_name)

        if os.path.exists(os.path.join(folder_name, file_name)):
            file_name = '{}_DUP_{}{}'.format(file_name, u.get_time_stamp(), idx)
        copy_file(file_item, os.path.join(folder_name, file_name))

def prune(target):
    """Removes duplicate files from the target"""

    files = u.find_all_files(target, ('.jpg', '.mp4', '.mov'))
    LOGGER.info('Scanning %s files...', len(files))

    to_remove = []
    for idx, primary_file in enumerate(files):
        for comparing_file in files[idx + 1: len(files)]:
            if filecmp.cmp(primary_file, comparing_file):
                LOGGER.info("duplicates found: %s and %s are equal; slating %s for removal",
                            primary_file, comparing_file, comparing_file)
                to_remove.append(comparing_file)

    LOGGER.info('Removing %s files...', len(to_remove))
    for file_item in to_remove:
        if os.path.isfile(file_item):
            LOGGER.info('removing file_item %s', file_item)
            os.remove(file_item)
        else:
            print('file_item {} previously removed'.format(file_item))

def fix_names(target):
    """ Checks that the name of the file_item matches the exif data
    contained in the file_item
    """
    files = u.find_all_files(target, ('.jpg', '.mp4', '.mov'))
    print('Scanning {} photos'.format(len(files)))
    for idx, file_item in enumerate(files):
        folder_name = os.path.join(target, u.generate_foldername_from_meta(file_item))
        file_name = u.generate_filename_from_meta(file_item)
        u.create_directory(folder_name)

        if os.path.exists(os.path.join(folder_name, file_name)):
            file_name = '{}_DUP_{}{}'.format(file_name, u.get_time_stamp(), idx)
        copy_file(file_item, os.path.join(folder_name, file_name))
        os.remove(file_item)

if __name__ == "__main__":
    pass



import click
import kurator.kurator as k

# -*- coding: utf-8 -*-
"""Example Google style docstrings.

This module demonstrates documentation as specified by the `Google Python
Style Guide`_. Docstrings may extend over multiple lines. Sections are created
with a section header and a colon followed by a block of indented text.

Example:
    Examples can be given using either the ``Example`` or ``Examples``
    sections. Sections support any reStructuredText formatting, including
    literal blocks::

        $ python example_google.py

Section breaks are created by resuming unindented text. Section breaks
are also implicitly created anytime a new section starts.

Attributes:
    module_level_variable1 (int): Module level variables may be documented in
        either the ``Attributes`` section of the module docstring, or in an
        inline docstring immediately following the variable.

        Either form is acceptable, but the two should not be mixed. Choose
        one convention to document module level variables and be consistent
        with it.

Todo:
    * For module TODOs
    * You have to also use ``sphinx.ext.todo`` extension

.. _Google Python Style Guide:
   http://google.github.io/styleguide/pyguide.html

"""
@click.group()
def cli():
    """Kurator helps manage photo and video dumps from a device
    to a photo library.

    A photo library is considered nothing more than a plain folder.
    """
    pass # pragma: no cover

@click.command()
@click.argument('source', type=click.Path(exists=True, file_okay=False, dir_okay=True))
@click.argument('library', type=click.Path(exists=True, file_okay=False, dir_okay=True))
def import_media(source, library):
    """ Imports media from a source into your library.

    Each image found in the source will be scanned for exif data.
    When copied to your library, it will be renamed using the date
    taken based on the exif data.

    Each image is placed into the library in a folder based on the date of its
    exif data.  If the folder doesn not already exist, it is created.

    If no exif data exists for the media file, a todays date combined with
    "NO_DATA" will be used as the filename.  A folder will be created using
    the same schema.

    SOURCE = the path of the media you want to import

    LIBRARY = the path of your photo library or any folder
    """
    k.import_media(source, library) # pragma: no cover


@click.command()
@click.argument('target', type=click.Path(exists=True, file_okay=False, dir_okay=True))
def prune(target):
    """Removes duplicate files from the target

    Each file in the target directory is compared to each other file recursively
    in the same directory.

    If two files are the same, the duplicate is removed.

    TARGET = the path of the media file folder
    """
    k.prune(target) # pragma: no cover

@click.command()
@click.argument('target', type=click.Path(exists=True, file_okay=False, dir_okay=True))
def fix_names(target):
    """ Rename all files in target using exif data (ie 20140201-165116.jpg)

    If no exif data exists for the media file, a todays date combined with
    "NO_DATA" will be used as the filename.

    TARGET = the path of the media file folder
    """
    k.fix_names(target) # pragma: no cover

def main():
    cli.add_command(import_media) # pragma: no cover
    cli.add_command(prune) # pragma: no cover
    cli.add_command(fix_names) # pragma: no cover
    cli() # pragma: no cover



# -*- coding: utf-8 -*-
"""Utilities

Utilities module

Todo:

.. _Google Python Style Guide:
   http://google.github.io/styleguide/pyguide.html
"""

from __future__ import print_function
import os
import time
import random
import hashlib

import exifread


def find_all_files(source, types):
    """Search for files of type and return list of absolute
    paths.

    source -- path to find files
    types -- tuple of types to search for
    """
    matches = []
    print(source)
    for root, _, files in os.walk(source):
        for filename in files:
            if filename.lower().endswith(types):
                matches.append(os.path.join(root, filename))
    return matches

def get_time_stamp():
    """Unix-like timestamp"""
    return str(int(time.time()))


def get_file_tags(path):
    """Get all exif tags from a file"""
    with open(path, 'rb') as file:
        raw_tags = exifread.process_file(file)
        tags = {
            'fileName': path,
            'dateTime': raw_tags['Image DateTime'].printable \
                if 'Image DateTime' in raw_tags \
                else 'NO_DATA_' + get_time_stamp() + str(random.randint(1, 1000)),
            'model': raw_tags['Image Model'].printable if 'Image Model' in raw_tags else 'unknown',
            'orientation': raw_tags['Image Orientation'].printable \
                if 'Image Orientation' in raw_tags \
                else 'unknown'
        }
        return tags

def generate_filename_from_meta(path):
    """Generate a filename from exif data.  Absolute path
    and extension included.
    """
    _, extension = os.path.splitext(path)
    return get_file_tags(path)['dateTime'].replace(':', '').replace(' ', '-') + extension.lower()

def generate_foldername_from_meta(path):
    """Generate folder name from exif data"""
    return get_file_tags(path)['dateTime'].replace(':', '').replace(' ', '-').lower()[:8]

def directory_exists(path):
    """Returns true if directory exists"""
    return os.path.isdir(path) and os.path.exists(path)

def create_directory(path):
    """Creates directory if it does not exist
    """
    if not directory_exists(path):
        print(path)
        os.makedirs(path)

def generate_md5(file):
    """Generate hashes"""
    return hashlib.md5(open(file, 'rb').read()).hexdigest()

def get_date_string_or(string):
    """If the string starts with 6 digits, that will be returned
    as a date in iso format.  Otherwise, todays date in iso
    format is returned.
    """
    if str(string[:5]).isdigit():
        return '{}-{}-{}'.format(string[:4], string[4:6], string[6:8])
    return time.strftime("%Y-%m-%d")

if __name__ == "__main__":
    pass



from setuptools import setup

setup(name='kurator',
      version='0.2',
      description='Command line tool to help with media dumps',
      keywords=['media','dump','file','transfer','exif','photos','images','duplicate','rename'],
      url='http://github.com/saltycatfish/kurator',
      download_url = 'https://github.com/saltycatfish/kurator/archive/0.2.tar.gz',
      author='SaltyCatFish',
      author_email='ryan@saltycatfish.com',
      license='MIT',
      packages=['kurator', 'kurator.lib'],
      install_requires=[
        'click==6.7',
        'ExifRead==2.1.2',
        'olefile==0.44',
        'Pillow==4.2.1',
        'psycopg2==2.7.3.1',
        'pyasn1==0.3.6',
        'pyasn1-modules==0.1.4',
        'rsa==3.4.2',
        'six==1.11.0',
        'uritemplate==3.0.0'
      ],
      entry_points = {
        'console_scripts': ['kurator=kurator.command_line:main'],
      },
      zip_safe=False)
